@@ -28,7 +28,7 @@ my $MAP_DESCR_FILE = "../nfdump/bin/nfx.c";
 my $MASTER_RECORD_FILE = "../nfdump/bin/nffile.h";
 
 # path to libnf C and H source files
-my $LIBNF_C_FILE = "libnf.c";
+my $LIBNF_C_FILE = "libnf_perl.c";
 
 
 # The perl structure parsed from 
@@ -105,15 +105,18 @@ sub get_master_record() {
 		next if (/#/);
 		next if (/^\s*$/);
 		# uint32_t	srcas;
-		if (/\s+(\w+_t)\s+(\w+)([\[\]\d]*);/) {
+#		printf "$_\n";
+		if (/\s+(\w+_t|char)\s+(\w+)([\[\]\d+]*);/) {
 			my ($type, $name, $arr) = ($1, $2, $3);
 
 			if (defined($arr)) {
 				$type .= $arr;
 			}
 
+		#	printf "$type | $name | $arr \n";
+
 			# ignore items like fill, any 
-			next if ($name =~ /^fill/ || $name =~ /^any$/ || $name =~ /^ext_map$/);
+			next if ($name =~ /fill/ || $name =~ /^any$/ || $name =~ /^ext_map$/);
 
 			push(@MASTER_RECORD, { 'name' => $name, 'type' => $type });
 		}
@@ -183,6 +186,7 @@ foreach (@MAP_DESCR) {
 	# skip "Required extension" and extension with no description 
 	next if ($_->{'description'} =~ /^Required extension:/);
 	next if ($_->{'description'} =~ /^$/);
+	next if ($_->{'description'} =~ /Compat NEL/);
 
 	my $read = str_count($LIBNF_C_FUNC{'libnf_master_record_to_SV'}, $_->{'id'}); 
 	my $write = str_count($LIBNF_C_FUNC{'libnf_write_row'}, $_->{'id'}); 
@@ -208,6 +212,7 @@ foreach (@MASTER_RECORD) {
 	next if ($_->{'name'} =~ /type|size/); 
 	next if ($_->{'name'} =~ /u\d{2}_\d/);
 	next if ($_->{'name'} =~ /exporter_sysid/); 
+	next if ($_->{'name'} =~ /nat_flags/); 
 
 	my $read = str_count($LIBNF_C_FUNC{'libnf_master_record_to_SV'}, $_->{'name'}); 
 	my $write = str_count($LIBNF_C_FUNC{'libnf_write_row'}, $_->{'name'}); 
